package com.qiaomeng.plugin.splash;

import android.content.Context;
import android.os.Handler;
import android.os.Looper;
import android.util.DisplayMetrics;
import android.view.Display;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.WindowManager;
import android.widget.ImageView;
import android.widget.TextView;

import com.bumptech.glide.Glide;
import com.bumptech.glide.GlideBuilder;
import com.bumptech.glide.disklrucache.DiskLruCache;
import com.bumptech.glide.load.engine.DiskCacheStrategy;
import com.bumptech.glide.load.engine.cache.DiskCache;
import com.bumptech.glide.load.engine.cache.SafeKeyGenerator;
import com.bumptech.glide.load.model.GlideUrl;
import com.bumptech.glide.request.RequestOptions;
import com.bumptech.glide.signature.EmptySignature;

import java.io.File;
import java.io.IOException;
import java.lang.reflect.Method;
import java.util.Timer;
import java.util.TimerTask;

public class Hanlder {
    private static Hanlder instance = new Hanlder();
    private Context context;
    private View layout;
    private DBUtils DB;
    private int seconds;
    private String imageUrl;
    private File imageFile;
    private Timer timer = new Timer();

    private OnClickListener onClickListener;
    private OnExposureListener onExposureListener;
    private OnCloseListener onCloseListener;

    public static Hanlder getInstance() {
        return instance;
    }

    public Hanlder init(Context context) {
        this.context = context;
        DB = new DBUtils(context);
        seconds = DB.getSeconds();
        Glide.init(context, new GlideBuilder());
        imageUrl = DB.getImage();
        if (imageUrl != null) {
            imageFile = getCacheFile(imageUrl);
        }
        return this;
    }

    private Hanlder() {

    }

    public void setMetadata(String metadata) {
        DB.setMetadata(metadata);
    }

    public void setImage(String url) {
        Glide.with(context).load(url).apply(new RequestOptions().diskCacheStrategy(DiskCacheStrategy.DATA)).preload();
        DB.setImage(url);
    }

    public void setSeconds(int seconds) {
        DB.setSeconds(seconds);
    }

    public void remove() {
        if (imageUrl == null || imageFile == null) {
            removeSplashScreen();
        }
    }

    public View createSplashScreen() {
        layout = LayoutInflater.from(context).inflate(R.layout.splash_screen_layout, null, false);
        final View logo = layout.findViewById(R.id.xx_logo);
        layout.post(new Runnable() {
            @Override
            public void run() {
                int screenHeight = getScreenHeight(context);
                int viewHeight = layout.getMeasuredHeight();
                int dis = screenHeight - viewHeight;
                if (dis > 0) {
                    logo.setTranslationY(dis);
                }

            }
        });
        if (imageUrl != null && imageFile != null) {
            int resourceId = context.getResources().getIdentifier("status_bar_height", "dimen", "android");
            int statusBarHeight = context.getResources().getDimensionPixelSize(resourceId);
            TextView closeBtn = layout.findViewById(R.id.xx_splash_close_btn);
            ImageView splashScreenImage = layout.findViewById(R.id.xx_splash_screen_image);
            closeBtn.setTranslationY(statusBarHeight);
            closeBtn.setVisibility(View.VISIBLE);
            splashScreenImage.setVisibility(View.VISIBLE);
            Glide.with(context).load(imageFile).apply(new RequestOptions().diskCacheStrategy(DiskCacheStrategy.DATA))
                    .into(splashScreenImage);
            downTime(closeBtn);
            closeBtn.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    removeSplashScreen();
                    if (onCloseListener != null) {
                        onCloseListener.onClose(DB.getMetadata());
                    }

                }
            });
            splashScreenImage.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    if (onClickListener != null) {
                        onClickListener.onClick(DB.getMetadata());
                        onClickListener = null;
                        removeSplashScreen();
                        if (onCloseListener != null) {
                            onCloseListener.onClose(DB.getMetadata());
                        }
                    }
                }
            });
            if (onExposureListener != null) {
                onExposureListener.onExposure(DB.getMetadata());
            }

        }

        return layout;
    }

    public void setOnClickListener(OnClickListener listener) {
        onClickListener = listener;
    }

    public void setOnExposureListener(OnExposureListener onExposureListener) {
        this.onExposureListener = onExposureListener;
    }

    public void setOnCloseListener(OnCloseListener onCloseListener) {
        this.onCloseListener = onCloseListener;
    }

    /**
     * 获取图片缓存地址
     *
     * @param url
     * @return
     */
    private File getCacheFile(String url) {
        DataCacheKey dataCacheKey = new DataCacheKey(new GlideUrl(url), EmptySignature.obtain());
        SafeKeyGenerator safeKeyGenerator = new SafeKeyGenerator();
        String safeKey = safeKeyGenerator.getSafeKey(dataCacheKey);
        try {
            int cacheSize = 100 * 1000 * 1000;
            DiskLruCache diskLruCache = DiskLruCache
                    .open(new File(context.getCacheDir(), DiskCache.Factory.DEFAULT_DISK_CACHE_DIR), 1, 1, cacheSize);
            DiskLruCache.Value value = diskLruCache.get(safeKey);
            if (value != null) {
                return value.getFile(0);
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
        return null;
    }

    private void downTime(final TextView closeBtn) {
        setSecondsText(closeBtn);
        TimerTask timerTask = new TimerTask() {
            @Override
            public void run() {
                closeBtn.post(new Runnable() {
                    @Override
                    public void run() {
                        seconds--;
                        setSecondsText(closeBtn);
                        if (seconds > 0) {
                            downTime(closeBtn);
                        } else {
                            removeSplashScreen();
                            if (onCloseListener != null) {
                                onCloseListener.onClose(DB.getMetadata());
                            }
                        }
                    }
                });
            }
        };
        timer.schedule(timerTask, 1000);
    }

    /**
     * 设置关闭按钮秒数
     *
     * @param closeBtn
     */
    private void setSecondsText(TextView closeBtn) {
        String text = "跳过 ";
        if (seconds > 0) {
            text += seconds;
        }
        closeBtn.setText(text);
    }

    /**
     * 移除启动页
     */
    private void removeSplashScreen() {
        this.timer.cancel();
        if (layout != null) {
            TimerTask timerTask = new TimerTask() {
                @Override
                public void run() {
                    Handler mainHandler = new Handler(Looper.getMainLooper());
                    mainHandler.post(new Runnable() {
                        @Override
                        public void run() {
                            if (layout.getParent() != null) {
                                ViewGroup parent = (ViewGroup) layout.getParent();
                                parent.removeView(layout);
                            }
                        }
                    });
                }
            };

            Timer timer = new Timer();
            timer.schedule(timerTask, 500);
        }
    }

    // 屏幕的全高
    private int getScreenHeight(Context context) {
        int dpi = 0;
        WindowManager windowManager = (WindowManager) context.getSystemService(Context.WINDOW_SERVICE);
        Display display = windowManager.getDefaultDisplay();
        DisplayMetrics displayMetrics = new DisplayMetrics();
        @SuppressWarnings("rawtypes")
        Class c;
        try {
            c = Class.forName("android.view.Display");
            @SuppressWarnings("unchecked")
            Method method = c.getMethod("getRealMetrics", DisplayMetrics.class);
            method.invoke(display, displayMetrics);
            dpi = displayMetrics.heightPixels;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return dpi;
    }

    public interface OnClickListener {
        void onClick(String metedata);
    }

    public interface OnExposureListener {
        void onExposure(String metedata);
    }

    public interface OnCloseListener {
        void onClose(String metedata);
    }
}
