package com.qiaomeng.plugin.splash;

import android.view.View;
import android.widget.RadioGroup;

import androidx.annotation.NonNull;
import io.flutter.embedding.android.FlutterActivity;
import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.embedding.engine.plugins.activity.ActivityAware;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;

/**
 * SplashScreenPlugin
 */
public class SplashScreenPlugin implements FlutterPlugin, MethodCallHandler, ActivityAware {
    private MethodChannel channel;
    private FlutterActivity flutterActivity;

    @Override
    public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
        channel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), "splash_screen_plugin");
        channel.setMethodCallHandler(this);
        Hanlder.getInstance().init(flutterPluginBinding.getApplicationContext());
        Hanlder.getInstance().setOnClickListener(new Hanlder.OnClickListener() {
            @Override
            public void onClick(String metedata) {
                channel.invokeMethod("click", metedata);
            }
        });
        Hanlder.getInstance().setOnExposureListener(new Hanlder.OnExposureListener() {
            @Override
            public void onExposure(String metedata) {
                channel.invokeMethod("exposure", metedata);
            }
        });

        Hanlder.getInstance().setOnCloseListener(new Hanlder.OnCloseListener() {
            @Override
            public void onClose(String metedata) {
                channel.invokeMethod("close", metedata);
            }
        });
    }

    @Override
    public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
        if (call.method.equals("setImage")) {
            String url = call.argument("imageUrl");
            Hanlder.getInstance().setImage(url);
        } else if (call.method.equals("setSeconds")) {
            if (call.hasArgument("seconds")) {
                int seconds = call.argument("seconds");
                Hanlder.getInstance().setSeconds(seconds);
            }
        } else if (call.method.equals("remove")) {
            Hanlder.getInstance().remove();
        } else if (call.method.equals("setMetadata")) {
            String metadata = call.argument("metadata");
            Hanlder.getInstance().setMetadata(metadata);
        } else if (call.method.equals("splash")) {
            if (flutterActivity != null) {
                View view = Hanlder.getInstance().init(flutterActivity).createSplashScreen();
                flutterActivity.addContentView(view, new RadioGroup.LayoutParams(RadioGroup.LayoutParams.MATCH_PARENT,
                        RadioGroup.LayoutParams.MATCH_PARENT));
            }
        } else {
            result.notImplemented();
        }
    }

    @Override
    public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
        channel.setMethodCallHandler(null);
    }

    @Override
    public void onAttachedToActivity(@NonNull ActivityPluginBinding binding) {
        flutterActivity = (FlutterActivity) binding.getActivity();
    }

    @Override
    public void onDetachedFromActivityForConfigChanges() {

    }

    @Override
    public void onReattachedToActivityForConfigChanges(@NonNull ActivityPluginBinding binding) {

    }

    @Override
    public void onDetachedFromActivity() {

    }
}
