import Flutter
import UIKit
import SnapKit
import Kingfisher

public class SwiftSplashScreenPlugin: NSObject, FlutterPlugin {
    
    var methodChannel:FlutterMethodChannel!
    
    var SplashView:UIView!
    
    var pluginMetadata:NSString!
    
    ///MARK: 定时器
    var skipText:UILabel!
    var timer: Timer?
    var limitTime: NSInteger = 0
    
    ///MARK 无广告
    var noADSplashView:UIView!
    
    
    
    public static let shared = SwiftSplashScreenPlugin()
    private override init() {}
    
    public static func register(with registrar: FlutterPluginRegistrar) {
        let channel = FlutterMethodChannel(name: "splash_screen_plugin", binaryMessenger: registrar.messenger())
        shared.initChannel(channel: channel)
        registrar.addMethodCallDelegate(shared, channel: channel)
    }
    
    func initChannel(channel:FlutterMethodChannel){
        self.methodChannel = channel
    }
    
    ///
    ///MARK 设置启动屏幕广告
    ///
    public func setSplashView(view:UIView) {
        let description:String = (object_getClass(view)?.description())!
        if(description != "FlutterView"){
            return
        }
        let userDefaults:UserDefaults = UserDefaults.standard
        let pluginSeconds = userDefaults.integer(forKey:"screen_plugin_seconds") as Any
        let imageUrl = userDefaults.string(forKey:"screen_plugin_image_url") as Any
        let metadata = userDefaults.string(forKey:"screen_plugin_metadata") as Any
        if object_getClass(metadata)?.description() != "NSNull" && object_getClass(pluginSeconds)?.description() != "NSNull" && object_getClass(imageUrl)?.description() != "NSNull" {
            self.pluginMetadata = (metadata as! NSString)
            self.SplashView = self.getSplashScreen(seconds:pluginSeconds as! NSInteger,imageUrl:imageUrl as! String)
            let splashDescription:String = (object_getClass(self.SplashView)?.description())!
            if(splashDescription == "UIView"){
                view.addSubview(SplashView)
                if methodChannel != nil{
                    methodChannel.invokeMethod("exposure", arguments:self.pluginMetadata)
                }
            }
        }else{
            let size:CGSize = UIScreen.main.bounds.size;
            let scale = size.width/375.0
            self.noADSplashView = UIView.init(frame:CGRect(x:0, y:0, width:size.width, height:size.height))
            self.noADSplashView.backgroundColor = .white
            
            let bannerView:UIView = UIView(frame:CGRect.zero)
            bannerView.backgroundColor = .white
            self.noADSplashView.addSubview(bannerView)
            bannerView.snp.makeConstraints { (make) in
                make.height.equalTo(size.width * 0.35)
                make.bottom.equalToSuperview()
                make.left.right.equalToSuperview()
            }
            
            let bannerImage:UIImageView = UIImageView(frame:CGRect.zero)
            bannerImage.image = UIImage(named:"img_open_logo@3x")
            bannerView.addSubview(bannerImage)
            bannerImage.snp.makeConstraints { make in
                make.height.equalTo(58 * scale)
                make.width.equalTo(190 * scale)
                make.centerX.equalToSuperview()
                make.centerY.equalToSuperview()
            }
            view.addSubview(self.noADSplashView)
        }
    }
    
    @objc func updateJumpBtn() {
        limitTime = limitTime - 1
        if (limitTime <= 0) {
            self.timer?.invalidate();
            self.SplashView.removeFromSuperview();
        }
        self.skipText.text = String(format:"跳过%ds",limitTime)
    }
    
    ///MARK 开屏广告栏
    public func getSplashScreen(seconds:NSInteger,imageUrl:String) -> UIView {
        self.limitTime = seconds
        self.timer = Timer.scheduledTimer(timeInterval:1.0, target: self, selector: #selector(updateJumpBtn), userInfo: nil, repeats: true)
        
        let resource:Resource = URL.init(string:imageUrl)!
        
        let size:CGSize = UIScreen.main.bounds.size;
        let scale = size.width/375.0
        
        let splash:UIView = UIView.init(frame:CGRect(x:0, y:0, width:size.width, height:size.height))
        splash.backgroundColor = .white
        
        ///MARK 广告图片
        let adView:UIView = UIView(frame:CGRect.zero)
        adView.backgroundColor = .white
        
        let adImageView:UIImageView = UIImageView(frame:CGRect.zero)
        adImageView.backgroundColor = .white
        //adImageView.contentMode =  .top
        adImageView.contentMode =  .scaleAspectFill
        //adImageView.clipsToBounds = true
        //adImageView.contentMode =  .scaleToFill
        //adImageView.contentMode =  .scaleAspectFit
        
        adView.addSubview(adImageView)
        adImageView.snp.makeConstraints { maker in
            maker.height.equalTo(size.height - size.width * 0.35)
            maker.left.right.top.equalToSuperview()
        }
        
        splash.addSubview(adView)
        adView.snp.makeConstraints { make in
            make.top.left.right.equalToSuperview()
            make.height.equalTo(size.height - size.width * 0.35)
        }
        
        KingfisherManager.shared.retrieveImage(with:resource) { result in
            switch result {
            case .success(let imgResult):
                adImageView.image = imgResult.image
                print("===============> 图片请求成功")
            case .failure(let error):
                print("============> 图片请求失败")
                print(error)
                break
            }
        }
        
        ///MARK 跳过
        let skipView:UIView = UIView(frame:CGRect.zero)
        skipView.layer.cornerRadius = 10*scale
        skipView.layer.masksToBounds = true
        skipView.backgroundColor = UIColor(red: 0.19, green: 0.15, blue: 0, alpha: 0.80)
        
        
        self.skipText = UILabel(frame:CGRect.zero)
        self.skipText.font = UIFont.systemFont(ofSize: 12)
        self.skipText.textColor = .white
        self.skipText.text = String(format:"跳过%ds",seconds)
        skipView.addSubview(self.skipText)
        self.skipText.snp.makeConstraints { make in
            make.center.equalToSuperview()
        }
        ///MARK 底部的Logo
        let bannerView:UIView = UIView(frame:CGRect.zero)
        bannerView.backgroundColor = .white
        splash.addSubview(bannerView)
        
        bannerView.snp.makeConstraints { (make) in
            make.height.equalTo(size.width * 0.35)
            make.bottom.equalToSuperview()
            make.left.right.equalToSuperview()
        }
        
        let bannerImage:UIImageView = UIImageView(frame:CGRect.zero)
        bannerImage.image = UIImage(named:"img_open_logo@3x")
        bannerView.addSubview(bannerImage)
        bannerImage.snp.makeConstraints { make in
            make.height.equalTo(58 * scale)
            make.width.equalTo(190 * scale)
            make.centerX.equalToSuperview()
            make.centerY.equalToSuperview()
        }
        
        splash.addSubview(skipView)
        skipView.snp.makeConstraints { make in
            make.size.equalTo(CGSize(width:82 * scale, height:32 * scale))
            make.top.equalToSuperview().offset(21 * scale)
            make.right.equalTo(splash.snp.right).offset(-13 * scale)
        }
        
        bannerView.addGestureRecognizer(UITapGestureRecognizer.init(target:self, action: #selector(tapBannerView(_:))))
        adView.addGestureRecognizer(UITapGestureRecognizer.init(target:self, action: #selector(tapAdView(_:))))
        skipView.addGestureRecognizer(UITapGestureRecognizer.init(target:self, action: #selector(tapSkipView(_:))))
        return splash
    }
    
    @objc func tapSkipView(_ view :UIView) {
        if self.pluginMetadata != nil  && methodChannel != nil {
            methodChannel.invokeMethod("close", arguments:self.pluginMetadata)
            if(self.SplashView != nil){
                self.SplashView.removeFromSuperview();
                self.timer?.invalidate();
            }
        }
    }
    
    @objc func tapAdView(_ view :UIView) {
        if self.pluginMetadata != nil && methodChannel != nil {
            methodChannel.invokeMethod("click", arguments:self.pluginMetadata)
            if(self.SplashView != nil){
                self.SplashView.removeFromSuperview();
                self.timer?.invalidate();
            }
        }
    }
    
    @objc func tapBannerView(_ view :UIView) {//"onExposure"
        print("=============>  底部Banner视图被点击   ")
    }
    
  public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
    let userDefaults:UserDefaults = UserDefaults.standard
    switch call.method {
    case "setImage":
        let arguments:NSDictionary = self.getArguments(arguments:call.arguments as Any);
        if let image_url:String = arguments["imageUrl"] as? String {
            let resource:Resource = URL.init(string:image_url)!
            KingfisherManager.shared.retrieveImage(with:resource) { result in
                switch result {
                case .success(_):
                    print("===============> 图片请求成功")
                case .failure(let error):
                    print("============> 图片请求失败")
                    print(error)
                    break
                }
            }
            userDefaults.setValue(image_url, forKey:"screen_plugin_image_url")
        }else{
            userDefaults.removeObject(forKey: "screen_plugin_image_url")
        }
    case "setSeconds":
        let arguments:NSDictionary = self.getArguments(arguments:call.arguments as Any);
        if let seconds:NSInteger = arguments["seconds"] as? NSInteger {
            userDefaults.setValue(seconds, forKey:"screen_plugin_seconds")
        }
    case "setMetadata":
        let arguments:NSDictionary = self.getArguments(arguments:call.arguments as Any);
        if let metadata:String = arguments["metadata"] as? String {
            userDefaults.setValue(metadata, forKey:"screen_plugin_metadata")
        }else{
            userDefaults.removeObject(forKey: "screen_plugin_metadata")
        }
    case "remove":
        if((self.noADSplashView) != nil){
            self.noADSplashView.removeFromSuperview();
        }
    default:
        result("iOS " + UIDevice.current.systemVersion)
    }
  }
    
    
    func getArguments(arguments:Any) -> NSDictionary {
        return (arguments as? NSDictionary)!
    }
}
